import { Button, Form, Input, Select, DatePicker, Radio, message, Modal, Cascader, InputNumber } from "antd";
import { useState, useMemo, useEffect } from "react";
import { api, useStaff } from "@nice/client";
import { areaOptions } from './area-options';
import InfoCard from './infoCard';
import DepartmentSelect from "@web/src/components/models/department/department-select";
import { baseFields } from "@web/src/app/admin/staffinfo-manage/defaultFields";
const StaffInfoWrite = () => {
  const [form] = Form.useForm();
  const [loading, setLoading] = useState(false);
  // 修改使用的hook方法
  const { create, setCustomFieldValue, useCustomFields } = useStaff();
  const { data: fields, isLoading: fieldsLoading } = useCustomFields();
  const [infoCards, setInfoCards] = useState<any[]>([]);

  // 添加跟踪培训和鉴定状态的state
  const [hasTrain, setHasTrain] = useState(false);
  const [hasCert, setHasCert] = useState(false);

  // 添加状态来跟踪每个文本区域的高度
  const [textAreaHeights, setTextAreaHeights] = useState<Record<string, number>>({});
  
  const handleAdd = (content: string[]) => {
    // 将数组内容展开为独立对象
    const newItems = content.map(text => ({ content: text }));
    setInfoCards(prev => [
      ...prev.filter(item => !content.includes(item.content)),
      ...newItems
    ]);
  }

  // 在组件中添加监听字段变化
  useEffect(() => {
    // 设置默认值
    form.setFieldsValue({
      hasTrain: false,
      hasCert: false
    });

    // 使用 Form 的 onValuesChange 在外部监听
    const fieldChangeHandler = () => {
      const values = form.getFieldsValue(['hasTrain', 'hasCert']);
      setHasTrain(!!values.hasTrain);
      setHasCert(!!values.hasCert);
    };

    // 初始化时执行一次
    fieldChangeHandler();

    // 不需要返回取消订阅，因为我们不再使用 subscribe
  }, [form]);

  // 按分组组织字段
  const fieldGroups = useMemo(() => {
    if (!fields) return {};
    return (fields as any[]).reduce((groups: any, field: any) => {
      const group = field.group || '其他信息';
      if (!groups[group]) {
        groups[group] = [];
      }
      groups[group].push(field);
      return groups;
    }, {});
  }, [fields]);

  // 检查字段是否应该被禁用的辅助函数
  const shouldFieldBeDisabled = (field: any, groupName: string) => {
    if (groupName === '培训信息' && !hasTrain) {
      return ['trainType', 'trainInstitute', 'trainMajor'].includes(field.name);
    }
    if (groupName === '鉴定信息' && !hasCert) {
      return ['certRank', 'certWork'].includes(field.name);
    }
    return false;
  };

  const renderField = (field: any, groupName: string) => {
    // 处理培训和鉴定的单选按钮
    if (field.name === 'hasTrain') {
      return (
        <Radio.Group
          options={[
            { label: '是', value: true },
            { label: '否', value: false }
          ]}
          onChange={(e) => {
            setHasTrain(e.target.value);
            form.setFieldsValue({ hasTrain: e.target.value });
          }}
          value={hasTrain}
        />
      );
    }

    if (field.name === 'hasCert') {
      return (
        <Radio.Group
          options={[
            { label: '是', value: true },
            { label: '否', value: false }
          ]}
          onChange={(e) => {
            setHasCert(e.target.value);
            form.setFieldsValue({ hasCert: e.target.value });
          }}
          value={hasCert}
        />
      );
    }

    // 检查字段是否应禁用
    const isDisabled = shouldFieldBeDisabled(field, groupName);


    // 根据字段类型渲染不同的组件
    switch (field.type) {
      case 'text':
        if(field.name === 'department'){
          return <DepartmentSelect disabled={isDisabled} />;
        }else{
          return <Input disabled={isDisabled} />;
        }
      case 'number':
        return <InputNumber disabled={isDisabled} />;
      case 'date':
        return <DatePicker disabled={isDisabled} />;
      case 'select':
        if (field.options && field.options.length > 0) {
          return <Select options={field.options} disabled={isDisabled} />;
        }
        return <Input placeholder="选项数据缺失" disabled={isDisabled} />;
      case 'radio':
        if (field.options && field.options.length > 0) {
          return <Radio.Group options={field.options} disabled={isDisabled} />;
        }
        return <Input placeholder="选项数据缺失" disabled={isDisabled} />;
      case 'textarea':
        return (
          <div
            className="w-full relative"
            style={{
              minHeight: textAreaHeights[field.name] ?
                `${textAreaHeights[field.name]}px` : 'auto'
            }}
          >
            <InfoCard 
              onAdd={(contents) => {
                handleAdd(contents);
                // 同步更新表单字段值
                form.setFieldValue(field.name, contents);
              }}
              onHeightChange={(height) => {
                setTextAreaHeights(prev => ({
                  ...prev,
                  [field.name]: height
                }));
              }}
            />
          </div>
        );
      case 'cascader':
        if (field.label === '籍贯') {
          return <Cascader options={areaOptions} disabled={isDisabled} />;
        } else {
          return <Input disabled={isDisabled} />;
        }
      default:
        return <Input className="w-full" disabled={isDisabled} />;
    }
  };
  const onFinish = async (e, values: any) => {
    // values.preventDefault();
    e.preventDefault()
    console.log(values)
    try {
      setLoading(true);
      // 创建基础员工记录
      if (!values.showname) {
        message.error("姓名不能为空");
        return;
      }
      // 创建基础员工记录
      console.log('准备创建用户，数据:', { showname: values.showname });
      const staff = await create.mutateAsync({
        data: {
          showname: values.showname,
          deptId: values.department? values.department : null,
        }
      });
      console.log('创建员工记录:', staff);
            // 创建系统日志记录
            await api.systemLog.create.mutateAsync({
              level: "info",
              module: "人员管理",
              action: "创建用户",
              targetId: staff.id,
              targetName: staff.username,
              message: `[${new Date().toLocaleString()}] 用户 ${staff.username} 的人员信息已成功添加`,
              details: {
                fields: validEntries.map(({ field, value }) => ({
                  name: field.label,
                  value
                }))
              },
              status: "success",
              departmentId: staff.deptId // 用户所属部门
            });
      // 过滤有效字段并转换值
      const validEntries = Object.entries(values)
        .filter(([key, value]) => 
          key !== 'showname' && // 新增排除 showname
          key !== 'username' && 
          key !== 'department' &&
          value !== undefined && 
          value !== null && 
          value !== ''
        )
        .map(([fieldName, value]) => {
          const field = fields && Array.isArray(fields) ? 
            fields.find((f: any) => f.name === fieldName) : undefined;
          
          // 处理特殊字段类型
          let processedValue = value;
          if (field?.type === 'date') {  //日期类型
            processedValue = (value as Date)?.toISOString();
          } else if (field?.type === 'cascader' && Array.isArray(value)) { //级联选择器
            processedValue = value?.join('/');
          }else if(field?.type === 'textarea'){ //多行文本
            processedValue = (value as string[])?.join(',');
          }

          return { field, value: processedValue };
        })
        .filter(item => item.field?.id);
  
      // 批量提交自定义字段
      await Promise.all(
        validEntries.map(({ field, value }) =>
          setCustomFieldValue.mutateAsync({
            staffId: staff.id,
            fieldId: field.id,
            value: String(value)
          })
        )
      );
      console.log('自定义字段提交成功', staff.username);
      // 记录系统日志 - 用户创建成功
      const timestamp = new Date().toLocaleString();
      const logs = [];
      // 记录用户创建
      logs.push(`${timestamp} - 用户创建成功：${staff.username}`);
      // 记录人员信息添加
      logs.push(`[${timestamp}] 用户 ${staff.username} 的人员信息已成功添加`);
      // 记录每个字段的详细信息
      validEntries.forEach(({ field, value }) => {
        if (field && field.label && value) {
          logs.push(`[${timestamp}] 提交的数据: ${field.label}=${value}`);
        }
      });
      // 根据字段分组记录
      const fieldsByGroup = validEntries.reduce((groups, { field, value }) => {
        if (field && field.group && value) {
          if (!groups[field.group]) {
            groups[field.group] = [];
          }
          groups[field.group].push({ field, value });
        }
        return groups;
      }, {});

      // 为每个分组记录信息
      Object.entries(fieldsByGroup).forEach(([groupName, fields]) => {
        const groupValues = (fields as any[]).map(f => `${f.field.label}=${f.value}`).join(', ');
        logs.push(`[${timestamp}] ${staff.username} 的${groupName}：${groupValues || '无'}`);
      });

      // 获取现有日志
      let currentLogs = [];
      try {
        const storedLogs = localStorage.getItem('systemLogs');
        currentLogs = storedLogs ? JSON.parse(storedLogs) : [];
      } catch (error) {
        console.error('读取系统日志失败', error);
      }

      // 添加新日志（倒序添加，最新的在最前面）
      const updatedLogs = [...logs.reverse(), ...currentLogs];

      // 保存到 localStorage
      localStorage.setItem('systemLogs', JSON.stringify(updatedLogs));

      // 如果有全局变量，也更新它
      if (typeof window !== 'undefined') {
        (window as any).globalLogs = updatedLogs;
      }

      message.success("信息提交成功");
      form.resetFields();
    } catch (error) {
      console.error('提交出错:', error);

      // 记录错误日志
      const timestamp = new Date().toLocaleString();
      const logMessage = `${timestamp} - 创建用户失败：${values.username || '未知用户'}, 错误: ${error.message || '未知错误'}`;

      // 获取现有日志
      let currentLogs = [];
      try {
        const storedLogs = localStorage.getItem('systemLogs');
        currentLogs = storedLogs ? JSON.parse(storedLogs) : [];
      } catch (err) {
        console.error('读取系统日志失败', err);
      }

      // 添加新日志
      const updatedLogs = [logMessage, ...currentLogs];

      // 保存到 localStorage
      localStorage.setItem('systemLogs', JSON.stringify(updatedLogs));

      // 如果有全局变量，也更新它
      if (typeof window !== 'undefined') {
        (window as any).globalLogs = updatedLogs;
      }

      message.error("提交失败，请重试");
    } finally {
      setLoading(false);
    }
  };

  if (fieldsLoading) {
    return <div>加载中...</div>;
  }

  return (
    <div className="max-w-4xl mx-auto p-6">
      <h1 className="text-2xl font-bold mb-6">人员信息管理</h1>

      <Form
        form={form}
        layout="vertical"
        onValuesChange={(changedValues) => {
          if ('hasTrain' in changedValues) {
            setHasTrain(!!changedValues.hasTrain);
          }
          if ('hasCert' in changedValues) {
            setHasCert(!!changedValues.hasCert);
          }
        }}
        className="space-y-6 mt-6"
      >
        {/* 新增基础信息区块 */}
      <div className="bg-blue-45 p-6 rounded-lg shadow relative mb-6">
        
        <div className="grid grid-cols-1 md:grid-cols-2 gap-x-6 gap-y-6">
          {baseFields.map((field) => (
            <Form.Item
              key={field.name}
              label={field.label}
              name={field.name}
              rules={[
                {
                  required: field.required,
                  message: `请输入${field.label}`,
                },
              ]}
            >
              {renderField(field, '系统默认信息')}
            </Form.Item>
          ))}
        </div>
      </div>
        {Object.entries(fieldGroups).map(([groupName, groupFields]) => (
          <div key={groupName} className="bg-white p-6 rounded-lg shadow relative">
            <h2 className="text-lg font-semibold mb-4">{groupName}</h2>
            <div className={`
              grid grid-cols-1 md:grid-cols-2 gap-x-6 gap-y-12
              ${groupName === "工作信息" ? "md:[&>*:nth-child(odd)]:col-start-1 md:[&>*:nth-child(even)]:col-start-2" : ""}
            `}>
              {(groupFields as any[])?.map((field: any) => (
                <Form.Item
                  key={field.id}
                  label={<span className="block mb-2">{field.label}</span>}
                  name={field.name}
                  rules={[
                    {
                      required: field.required,
                      message: `请输入${field.label}`,
                    },
                  ]}
                  className={`
                    ${field.type === 'textarea' && groupName !== "工作信息" ? "md:col-span-2" : ""}
                    ${field.type === 'textarea' ? "transition-all duration-200 ease-in-out" : ""} 
                  `}
                  style={{
                    minHeight: field.type === 'textarea' && textAreaHeights[field.name] ?
                      `${textAreaHeights[field.name] + 50}px` : 'auto'
                  }}
                >
                  {renderField(field, groupName)}
                </Form.Item>
              ))}
            </div>
          </div>
        ))}

        <div className="flex justify-end space-x-4">
          <Button onClick={() => form.resetFields()}>重置</Button>
          <Button
            type="primary"
            htmlType="submit"
            onClick={(e) => onFinish(e, form.getFieldsValue())}
            loading={loading}
          >
            提交
          </Button>
        </div>
      </Form>
    </div>
  );
};

export default StaffInfoWrite;